/* 
 * Secrets.java
 *
 * Copyright 2003, Leigh L. Klotz, Jr..  All Rights Reserved.
 *   Email: klotz@graflex.org
 *     Web: http: //www.graflex.org/klotz
 *
 * {LICENSE}
 */
package com.hipme.secrets;

import danger.app.*;
import danger.app.Registrar;
import danger.app.IPCIncoming;
import danger.app.IPCMessage;
import danger.ui.*;
import danger.net.*;
import danger.util.*;
import danger.util.format.StringFormat;
import danger.util.DEBUG;

import java.util.Vector;

/**
 * Implements the main application class.
 *
 */
public class Secrets extends Application implements Resources, Events {
  final static String DATASTORE_NAME = "SECRETS";
  MainWindow mainWindow;

  /**
   * Constructor
   */
  public Secrets() {
    System.err.println( "Secrets: starting up" ); 
    DataStore ds = getDataStore();
    if (ds == null) {
      DEBUG.p("Secrets: creating data store");
      ds = DataStore.createDataStore(DATASTORE_NAME, true);
    }
  }
	
  /*
   * Listener API
   */
  public boolean receiveEvent(Event event) {
    boolean consumed = true;
    switch (event.type) {
    case kCmdHelp:
      mainWindow.help(); 
      break;
    case kCmdNew:
      mainWindow.showCellInfo(); 
      break;
    case kUpdateList:
      mainWindow.sendUpdate(); 
      break;
    case kCmdShowAll:
      mainWindow.showAllTowers(event); 
      break;
    case kCmdShowMatching:
      mainWindow.showAllTowers(event); 
      break;
    case kCmdLock:
      mainWindow.lock();
      break;
    case kCmdShowUnlock:
      mainWindow.showUnlock();
      break;
    case kCmdUnlock:
      mainWindow.unlock();
      break;
    case kShowDiscardEntry:
      mainWindow.showDiscardEntry();
      break;
    case EVENT_DATASTORE_RESTORED:
      dataStoreRestored();
      break;
    case kReturnToLauncher: 
      returnToLauncher();
      break;
    default: 
      consumed = false;
    }
    System.err.println("Got event e.type"+event.type+" event="+event);
    event.debugPrint();
    return consumed || super.receiveEvent(event);
  }

  /**
   * Application API
   */
  public void resume() {
    // Just create the window, if it does not already exist
    if (mainWindow == null) {
      // Create a new window and bring it to the foreground
      mainWindow = new MainWindow(this);
      mainWindow.setTitle(getString(kStrAppName));
      mainWindow.show();
    } else 
      mainWindow.sendUpdate();
    super.resume();
  }

  public void suspend() {
    // start timeout thread to lock?
    super.suspend();
  }

  public final void networkEvent(Object s) {
    System.err.println("Got networkEvent " + s);
    if (s instanceof HTTPTransaction) {
      HTTPTransaction t = (HTTPTransaction) s;
      if (t.getResponse() == 200) {
	mainWindow.handleResponse(t);
      }
      else {
	new AlertWindow("Error: "+t.getResponseMessage()+"\n"+
			t.getString()).show();
      }
    }
  }

  DataStore getDataStore() {
    return DataStore.findDataStore(DATASTORE_NAME);
  }

}
