/* 
 *
 * Copyright 2003, Leigh L. Klotz, Jr..  All Rights Reserved.
 *   Email: klotz@graflex.org
 *     Web: http: //www.graflex.org/klotz
 *
 * {LICENSE}
 */
package com.hipme.secrets;

import danger.app.*;
import danger.audio.Meta;
import danger.ui.*;
import danger.net.*;
import danger.util.*;
import danger.util.format.StringFormat;
import danger.util.DEBUG;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

class MainWindow extends ScreenWindow implements Resources, Events {
  Secrets app;
  SecretListView listView;

  MainWindow(Secrets app) {
    this.app = app;
    Rect bounds = getBounds();
    int width = (bounds.right - bounds.left);
    int height = (bounds.bottom - bounds.top);
	    
    ResourceDatabase db = app.getResources();
    Menu menu = getActionMenu();
    menu.removeAllItems();
    menu.addFromResource(db, kSecretsMenu);

    listView = new SecretListView();
    listView.setSize(width, height);
    listView.setPosition(0, 0);
    addChild(listView);
    showAllChildren();
    setFocusedChild(listView);
    sendUpdate();
    DEBUG.p("secrets: mainwindow done");
  }
	
  void sendUpdate() {
    new Event(this, kUpdateList).sendToTarget();
  }

  // now we need locking...
  private void update() {
    DEBUG.p("secrets: update");
    DataStore ds = app.getDataStore();
    if (ds == null) return;
    int nRecords = ds.getRecordCount();
    listView.removeAllItems();
    Vector v = Record.getSecrets();
    for (int i = 0; i < nRecords; i++) {
      Record record = new Record(ds.getRecordData(i));
      int recordUID = ds.getRecordUID(i);
      record.setRecordUID(recordUID);
      MenuItem m = listView.addItem(record.getTitle(), this, kShowEditEntry, recordUID, record);
    }
    if (nRecords > 0) 
      listView.setSelection(nRecords-1);
    listView.invalidate();
    DEBUG.p("secrets: update done");
  }

  public boolean receiveEvent(Event event) {
    boolean consumed = true;
    switch (event.type) {
    case kShowAddEntry:
      showAddEntry(event);
      break;
    case kAddNewEntry:
      handleAddEntry(event);
      break;
    case kShowEditEntry:
      showEditEntry(event);
      break;
    case kEditEntry:
      handleAddEntry(event);
      break;
    case kShowDiscardEntry:
      showDiscardEntry(event);
      break;
    case kLockAndReturnToLauncher:
      handleLockAndReturnToLauncher();
      break;
    case kDiscardEntry:
      handleDiscardEntry(event);
      break;
    default:
      consumed = super.receiveEvent(event);
    }
    return consumed;
  }

  public boolean eventWidgetDown(int i, Event event) {
    boolean consumed = true;
    switch(i) {
    case Event.DEVICE_BUTTON_BACK:
      // X locks and exits app
      handleLockAndReturnToLauncher();
      break;
    default:
      consumed = super.eventWidgetDown(i, event);
    }
    return consumed;
  }

  public final boolean eventKeyUp(char i, Event event) {
    boolean consumed = false;
    if (i == 8) {
      showDiscardEntry();
      consumed = true;
    }
    if (! consumed) 
      consumed = super.eventKeyUp(i, event);
    return consumed;
  }

  void sendExitApp() {
    new Event(Application.getCurrentApp(), kReturnToLauncher).sendToTarget();
  }

  void handleLockAndReturnToLauncher() {
    lockData();
    new Event(Application.getCurrentApp(), kReturnToLauncher).sendToTarget();
  }

  void help() {
    new AlertWindow("Help!").show();
  }
	
 void handleEditEntry(Event event) {
    MyDialogWindow dw = ((MyDialogWindow)(event.argument));
    Record record = new Record();
    record.parseDialogWindow(dw);
    int idx = ds.addRecord(record.getBytes());
    sendUpdate();
  }

  void handleDiscardEntry(Event event) {
    // it's too hard to pass the item from showDiscardEntry to discardEntry through the dialog
    // so we just look it up again and hope hope there's no way to move the cursor
    MenuItem item = listView.getSelectedItem();
    if (item == null) {
      Meta.play(Meta.DELETE_ITEM);
      return;
    }
    Record record = ((Record)(item.getEventArg()));
    String uid = record.getUID();
    int recordUID =  item.getEvent().data;
    deleteEntry(uid, recordUID);
  }

  void deleteEntry(String uid, int recordUID) {
    DataStore ds = app.getDataStore();
    DEBUG.p("deleteEntry(uid="+uid+", recordUID="+recordUID);
    // we have to try removing twice because
    // it doesn't seem to work to remove 
    ds.removeRecordByUID(recordUID);
    // records with the wrong sign
    // and the sign changes when the record is sync'd.
    ds.removeRecordByUID(-recordUID);
  }

  void showAddEntry() {
    showEntry(new Record());
  }

  void showEditEntry(Event event) {
    showEntry(((Record)(event.argument)));
  }

  void showDiscardEntry() {
    MenuItem item = listView.getSelectedItem();
    if (item == null) {
      Meta.play(Meta.DELETE_ITEM);
      return;
    }
    Record record = ((Record)(item.getEventArg()));
    MyDialogWindow dw = ((MyDialogWindow)(app.getDialog(kSecretsDiscardDialog, this)));
    dw.setStringValue(kSecretsDiscardDialogText, record.getTitle());
    dw.show();
  }

  void showEntry(Record record) {
    MyDialogWindow dw = ((MyDialogWindow)(app.getDialog(kSecretsDialog, this)));
    if (record.hasUID()) 
      dw.setStringValue(kSecretsUID, record.getUID());
    if (record.hasRecordUID()) 
      dw.setStringValue(kSecretsRecordUID, ""+record.getRecordUID());
    dw.setStringValue(kSecretsOper, record.getOPER());
    dw.setStringValue(kSecretsLAC, record.getLAC());
    dw.setStringValue(kSecretsCID, record.getCID());
    dw.setStringValue(kSecretsSignalStrength, record.getSignalStrength());
    dw.setStringValue(kSecretsASU, "" + record.getASU());
    dw.setStringValue(kSecretsDirection, towerStrings [record.getSector()]);
    dw.setStringValue(kSecretsName, record.getName());
    dw.setStringValue(kSecretsAddress, record.getAddress());
    dw.setStringValue(kSecretsCity, record.getCity());
    dw.setStringValue(kSecretsState, record.getState());
    dw.setStringValue(kSecretsZip, record.getZip());
    dw.setStringValue(kSecretsLatitude, record.getLatitude());
    dw.setStringValue(kSecretsLongitude, record.getLongitude());
    dw.setBooleanValue(kSecretsIsTowerLocation, record.isTowerLocation());
    dw.setBooleanValue(kSecretsIsIndoors, record.isIndoors());
    {
      dw.hideText(kSecretsBingo);
      dw.hideText(kSecretsHot);
      if (record.getASU() >= BINGO_THRESHOLD_ASU)
	dw.showText(kSecretsBingo);
      else if (record.getASU() >= HOT_THRESHOLD_ASU)
	dw.showText(kSecretsHot);
    }
    dw.show();
  }

}

